from madap.logger import logger
from plotting import Plots


log = logger.get_logger("voltammetry_plotting")

class VoltammetryPlotting(Plots):
    # TODO add corresponding methods to this class
    pass

    # TODO columbic_efficiency vs time
    # TODO Q vs time (capacity(mAh))

    # CP
    # TODO V vs t
    # TODO dQ vs dV
    # TODO v vs Q

    # CA
    # TODO i vs t
    # TODO i vs Q
    # TODO currentDensity vs time
    # TODO Q vs cycle

    #cv
    # TODO plots
    # TODO V vs I